==> ask-cli-1.5.0.tgz: package/lib/dialog/dialog.js <==
'use strict';

const jsonUtility = require('../utils/json-utility');
const validator = require('./dialog-validator');
const fs = require('fs');
const sender = require('./simulation-sender');
const inquirer = require('inquirer');
const promiseUtility = require('../utils/promise-while');
const chalk = require('chalk');
const asyncWriter = require('./file-writer');
const parser = require('./response-parser');
const isValidFilePath = require('is-valid-path');
const ResponseTypeEnum = Object.freeze({CAPTIONS: "Captions", ERROR: "Error"});
let isDialogEnded = false;
let shouldForceNewSession = true;
let outputWriteStream = null;
let outputFileName = null;
let recordInputArray = [];
let inputTextArray = [];
const specialCommandsMap = {
    '!quit' : true,
    '!record' : true,
};

module.exports = {
    createCommand: commander => {
        commander
            .command('dialog')
            .usage('[-s|--skill-id <skill-id>] [-l|--locale <locale>] [-r|--replay <file-path>] [-o|--output <file-name>] [-p|--profile <profile>] [--debug]')
            .description('(Beta)simulate your skill via interactive dialog with Alexa')
            .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
            .option('-l, --locale <locale>', 'locale for the utterance text')
            .option('-o, --output <file-path>', 'log the interactive session into a file')
            .option('-r, --replay <file-path>', 'input file that contains a recorded session')
            .option('-p, --profile <profile>', 'ask cli profile')
            .option('--debug', 'ask cli debug mode')
            .on('--help', function() {
                console.log('\r\n  Special Commands:');
                console.log('\r\n    !record    record the user transcript');
                console.log('    !quit      quit interactive mode');
            })
            .action(handle);
    }
};

// TODO refactor code combaine multiple function into one dialog module
function handle(options) {
    const validOptions = validator.validateInputOption(options);
    const {skillId, locale, replay, profile, debug} = validOptions;
    outputFileName = validOptions.output;
    

    // Call SMAPI to check skill ownership to:
    // 1.Exit dialog mode quickly if check failed.
    // 2.Get rid of client error during dialog CLI.
    validator.validateUserInputSkillInfo(skillId, locale, profile, debug)
    .then(() => {
        if (outputFileName) {
            outputWriteStream = fs.createWriteStream(outputFileName, { flags: 'w' });
            outputWriteStream.on('error', (error) => {
                // No exit for write stream error
                console.error(`[Fatal]: Failed to create the write stream for the output file. ${error}`);
            });
        }
        if (replay) {    
            console.log(`  [Info]: Replaying file ${replay}.`);
            const replayUserInputs = jsonUtility.getProperty(replay, ['userInput']);
            replayConversation(replayUserInputs, skillId, locale, profile, debug).then(() => {
                promiseUtility.promiseWhile(function() {
                    return !isDialogEnded;
                }, function() {
                    return startConversation(skillId, locale, profile, debug);
                });                
            });
        } else {
            promiseUtility.promiseWhile(function() {
                return !isDialogEnded;
            }, function() {
                return startConversation(skillId, locale, profile, debug);
            });            
        }
    })
    .catch(err => {
        if (err) {
            console.error('[Error]:', err);
        }
        return process.exit(1);
    });
}

function createUserInputsRequest(userInputs, skillId, locale, profile, debug) {
    return userInputs.map((item) => (sessionMode) => {
        return new Promise((resolve, reject) => {
            console.log(chalk.bold.yellow('  User  > '), item);
            if (specialCommandsMap.hasOwnProperty(item.toLowerCase())) {
                console.error('  [Error]: Replay file should not include any special command.');
                return process.exit(1);
            }
                
            return handleUtterance(item, skillId, locale, sessionMode, profile, debug)
            .then(() => resolve())
            .catch(() => reject());
        });
    });
}

function replayConversation(userInputs, skillId, locale, profile, debug) {
    return new Promise((resolve) => {
        createUserInputsRequest(userInputs, skillId, locale, profile, debug)
            .reduce((promiseChain, func, itemIndex) => {
                return promiseChain.then(() => {
                    if (itemIndex === 0) {
                        let needNewSession = shouldForceNewSession;
                        shouldForceNewSession = false;
                        return func(needNewSession);
                    } else {
                        return func();
                    }
                });
            }, Promise.resolve())
            .catch(err => {
                resolve();
            })
            .then(() => resolve());
    });
}

function startConversation(skillId, locale, profile, debug) {
    return new Promise(function(resolve, reject) {
        const questions = {
            type: 'input',
            name: 'userSays',
            message: chalk.yellow(' User  > '),
            prefix: '',
            validate: (value) => {
                if (typeof value !== 'string' || !value.trim()) {
                    return 'Please enter a non-empty text.';
                } if (value.startsWith('!') && (!specialCommandsMap.hasOwnProperty(value.toLowerCase())) ) {
                    return 'Invalid command. Command option can only be "!record" or "!quit".';
                } else {
                    return true;
                }
            }
        };

        inquirer.prompt(questions).then(answers => {
            let userInput = answers.userSays;
            if (userInput.startsWith('!')) {
                handleSpecialCommands(userInput, skillId, locale)
                .then(() => resolve());
            } else {
                const localeForceNewSession = shouldForceNewSession;
                shouldForceNewSession = false;
                // Treat error case like the normal case, just show the error message.
                handleUtterance(userInput, skillId, locale, localeForceNewSession, profile, debug)
                .then(() => resolve())
                .catch(() => resolve());
            }
        });
    });
}

function showResponses(response, responseType) {
    switch(responseType) {
        case ResponseTypeEnum.CAPTIONS:
            showCaptions(response);
            break;
        case ResponseTypeEnum.ERROR:
            console.error('  [Error]:', response);
            break;
        default:
            // should not reached.
            console.error('[Fatal]: responseType Invalid.');
    }
}

function showCaptions(response) {
    const requestType = jsonUtility.getPropertyValueFromObject(response, 
        ['result', 'skillExecutionInfo', 'invocationRequest', 'body', 'request', 'type']
    );
    
    if (requestType && requestType === 'SessionEndedRequest') {
        const sessionEndReason = jsonUtility.getPropertyValueFromObject(response, 
            ['result', 'skillExecutionInfo', 'invocationRequest', 'body', 'request', 'reason']
        );
        console.log(`  [Info]: Skill session ended becuase skill received SessionEndedRequest with reason ${sessionEndReason}`);
        resetSession();
    }

    const captions = jsonUtility.getPropertyValueFromObject(
        response, ['result', 'alexaExecutionInfo', 'alexaResponses']
    );
    if (captions) {
        captions.forEach(item => {
            console.log(chalk.bold.yellow('  Alexa > '), item.content.caption.trim());
        });
    }
}

function filePathPrompt(callback) {
    const filePathQuestions = {
        type: 'input',
        name: 'filePath',
        message: 'Enter the file name you want to record to:',
        validate: (value) => {
            if (typeof value !== 'string' || !value.trim()) {
                return 'Please enter a non-empty file name.';
            } else if (!isValidFilePath(value)){
                return 'Please enter a valid file name or file path.';
            } else if (value === outputFileName) {
                return `${value} already used for output file name, please input a new record file name.`;
            } else {
                return true;
            }
        }
    };
    inquirer.prompt(filePathQuestions).then(answers => {
        callback(answers.filePath);
    });
}

function handleSpecialCommands(userInput, skillId, locale) {
    return new Promise(resolve => {
        const specialInput = userInput.toLowerCase();
        if (specialInput === '!quit') {
            process.nextTick(() => {
                isDialogEnded = true;
                resolve();
            });
        } else if (specialInput === '!record') {
            filePathPrompt(userFileNameInput => {
                const userRecordObject = {
                    skillId: skillId,
                    locale: locale,
                    type: 'text',
                    userInput: recordInputArray
                };
                jsonUtility.write(userFileNameInput, userRecordObject);
                isDialogEnded = false;
                resolve();          
            });        
        } 
    });
}

function handleUtterance(userInput, skillId, locale, sessionMode, profile, debug) {
    return new Promise((resolve,reject) => {
        sender
        .sendDialogSimulate(userInput, skillId, locale, sessionMode, outputWriteStream, inputTextArray, profile, debug)
        .then((response) => {
            isDialogEnded = false;
            recordInputArray = inputTextArray;

            showResponses(response, ResponseTypeEnum.CAPTIONS);
            if (parser.isSkillResponseSessionEnd(response)) {
                console.log('---------- Skill Session Ended ----------');
                resetSession();
            }
            if (outputWriteStream) {
                asyncWriter.writeResponses(response, outputWriteStream);
            }
            resolve();
        })
        .catch(errorResponse => {
            recordInputArray = inputTextArray;

            showResponses(errorResponse.msg, ResponseTypeEnum.ERROR);
            if (outputWriteStream) {
                asyncWriter.writeError(errorResponse.payload, outputWriteStream);
            }
            resetSession();
            reject();
        });
    });
}

// When 1.error occurred, 2.skill session ended, we set shouldForceNewSession to ture 
// to make sure next utterance start a new skill session.
function resetSession() {
    inputTextArray = [];
    shouldForceNewSession = true;
}

==> ask-cli-1.6.0.tgz: package/lib/dialog/dialog.js <==
'use strict';

const jsonUtility = require('../utils/json-utility');
const validator = require('./dialog-validator');
const fs = require('fs');
const sender = require('./simulation-sender');
const inquirer = require('inquirer');
const promiseUtility = require('../utils/promise-while');
const chalk = require('chalk');
const asyncWriter = require('./file-writer');
const parser = require('./response-parser');
const isValidFilePath = require('is-valid-path');
const ResponseTypeEnum = Object.freeze({CAPTIONS: "Captions", ERROR: "Error"});
let isDialogEnded = false;
let shouldForceNewSession = true;
let outputWriteStream = null;
let outputFileName = null;
let recordInputArray = [];
let inputTextArray = [];
const specialCommandsMap = {
    '!quit' : true,
    '!record' : true,
};

module.exports = {
    createCommand: commander => {
        commander
            .command('dialog')
            .usage('[-s|--skill-id <skill-id>] [-l|--locale <locale>] [-r|--replay <file-path>] [-o|--output <file-name>] [-p|--profile <profile>] [--debug]')
            .description('(Beta)simulate your skill via interactive dialog with Alexa')
            .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
            .option('-l, --locale <locale>', 'locale for the utterance text')
            .option('-o, --output <file-path>', 'log the interactive session into a file')
            .option('-r, --replay <file-path>', 'input file that contains a recorded session')
            .option('-g, --stage <stage>', 'stage for the skill')
            .option('-p, --profile <profile>', 'ask cli profile')
            .option('--debug', 'ask cli debug mode')
            .on('--help', function() {
                console.log('\r\n  Special Commands:');
                console.log('\r\n    !record    record the user transcript');
                console.log('    !quit      quit interactive mode');
            })
            .action(handle);
    }
};

// TODO refactor code combaine multiple function into one dialog module
function handle(options) {
    const validOptions = validator.validateInputOption(options);
    const {skillId, locale, replay, stage, profile, debug} = validOptions;
    outputFileName = validOptions.output;


    // Call SMAPI to check skill ownership to:
    // 1.Exit dialog mode quickly if check failed.
    // 2.Get rid of client error during dialog CLI.
    validator.validateUserInputSkillInfo(skillId, locale, stage, profile, debug)
    .then(() => {
        if (outputFileName) {
            outputWriteStream = fs.createWriteStream(outputFileName, { flags: 'w' });
            outputWriteStream.on('error', (error) => {
                // No exit for write stream error
                console.error(`[Fatal]: Failed to create the write stream for the output file. ${error}`);
            });
        }
        if (replay) {    
            console.log(`  [Info]: Replaying file ${replay}.`);
            const replayUserInputs = jsonUtility.getProperty(replay, ['userInput']);
            replayConversation(replayUserInputs, skillId, locale, stage, profile, debug).then(() => {
                promiseUtility.promiseWhile(function() {
                    return !isDialogEnded;
                }, function() {
                    return startConversation(skillId, locale, stage, profile, debug);
                });                
            });
        } else {
            promiseUtility.promiseWhile(function() {
                return !isDialogEnded;
            }, function() {
                return startConversation(skillId, locale, stage, profile, debug);
            });            
        }
    })
    .catch(err => {
        if (err) {
            console.error('[Error]:', err);
        }
        return process.exit(1);
    });
}

function createUserInputsRequest(userInputs, skillId, locale, stage, profile, debug) {
    return userInputs.map((item) => (sessionMode) => {
        return new Promise((resolve, reject) => {
            console.log(chalk.bold.yellow('  User  > '), item);
            if (specialCommandsMap.hasOwnProperty(item.toLowerCase())) {
                console.error('  [Error]: Replay file should not include any special command.');
                return process.exit(1);
            }
                
            return handleUtterance(item, skillId, locale, sessionMode, stage, profile, debug)
            .then(() => resolve())
            .catch(() => reject());
        });
    });
}

function replayConversation(userInputs, skillId, locale, stage, profile, debug) {
    return new Promise((resolve) => {
        createUserInputsRequest(userInputs, skillId, locale, stage, profile, debug)
            .reduce((promiseChain, func, itemIndex) => {
                return promiseChain.then(() => {
                    if (itemIndex === 0) {
                        let needNewSession = shouldForceNewSession;
                        shouldForceNewSession = false;
                        return func(needNewSession);
                    } else {
                        return func();
                    }
                });
            }, Promise.resolve())
            .catch(err => {
                resolve();
            })
            .then(() => resolve());
    });
}

function startConversation(skillId, locale, stage, profile, debug) {
    return new Promise(function(resolve, reject) {
        const questions = {
            type: 'input',
            name: 'userSays',
            message: chalk.yellow(' User  > '),
            prefix: '',
            validate: (value) => {
                if (typeof value !== 'string' || !value.trim()) {
                    return 'Please enter a non-empty text.';
                } if (value.startsWith('!') && (!specialCommandsMap.hasOwnProperty(value.toLowerCase())) ) {
                    return 'Invalid command. Command option can only be "!record" or "!quit".';
                } else {
                    return true;
                }
            }
        };

        inquirer.prompt(questions).then(answers => {
            let userInput = answers.userSays;
            if (userInput.startsWith('!')) {
                handleSpecialCommands(userInput, skillId, locale)
                .then(() => resolve());
            } else {
                const localeForceNewSession = shouldForceNewSession;
                shouldForceNewSession = false;
                // Treat error case like the normal case, just show the error message.
                handleUtterance(userInput, skillId, locale, localeForceNewSession, stage, profile, debug)
                .then(() => resolve())
                .catch(() => resolve());
            }
        });
    });
}

function showResponses(response, responseType) {
    switch(responseType) {
        case ResponseTypeEnum.CAPTIONS:
            showCaptions(response);
            break;
        case ResponseTypeEnum.ERROR:
            console.error('  [Error]:', response);
            break;
        default:
            // should not reached.
            console.error('[Fatal]: responseType Invalid.');
    }
}

function showCaptions(response) {
    const requestType = jsonUtility.getPropertyValueFromObject(response, 
        ['result', 'skillExecutionInfo', 'invocationRequest', 'body', 'request', 'type']
    );
    
    if (requestType && requestType === 'SessionEndedRequest') {
        const sessionEndReason = jsonUtility.getPropertyValueFromObject(response, 
            ['result', 'skillExecutionInfo', 'invocationRequest', 'body', 'request', 'reason']
        );
        console.log(`  [Info]: Skill session ended becuase skill received SessionEndedRequest with reason ${sessionEndReason}`);
        resetSession();
    }

    const captions = jsonUtility.getPropertyValueFromObject(
        response, ['result', 'alexaExecutionInfo', 'alexaResponses']
    );
    if (captions) {
        captions.forEach(item => {
            console.log(chalk.bold.yellow('  Alexa > '), item.content.caption.trim());
        });
    }
}

function filePathPrompt(callback) {
    const filePathQuestions = {
        type: 'input',
        name: 'filePath',
        message: 'Enter the file name you want to record to:',
        validate: (value) => {
            if (typeof value !== 'string' || !value.trim()) {
                return 'Please enter a non-empty file name.';
            } else if (!isValidFilePath(value)){
                return 'Please enter a valid file name or file path.';
            } else if (value === outputFileName) {
                return `${value} already used for output file name, please input a new record file name.`;
            } else {
                return true;
            }
        }
    };
    inquirer.prompt(filePathQuestions).then(answers => {
        callback(answers.filePath);
    });
}

function handleSpecialCommands(userInput, skillId, locale) {
    return new Promise(resolve => {
        const specialInput = userInput.toLowerCase();
        if (specialInput === '!quit') {
            process.nextTick(() => {
                isDialogEnded = true;
                resolve();
            });
        } else if (specialInput === '!record') {
            filePathPrompt(userFileNameInput => {
                const userRecordObject = {
                    skillId: skillId,
                    locale: locale,
                    type: 'text',
                    userInput: recordInputArray
                };
                jsonUtility.write(userFileNameInput, userRecordObject);
                isDialogEnded = false;
                resolve();          
            });        
        } 
    });
}

function handleUtterance(userInput, skillId, locale, sessionMode, stage, profile, debug) {
    return new Promise((resolve,reject) => {
        sender
        .sendDialogSimulate(userInput, skillId, locale, sessionMode, outputWriteStream, inputTextArray, stage, profile, debug)
        .then((response) => {
            isDialogEnded = false;
            recordInputArray = inputTextArray;

            showResponses(response, ResponseTypeEnum.CAPTIONS);
            if (parser.isSkillResponseSessionEnd(response)) {
                console.log('---------- Skill Session Ended ----------');
                resetSession();
            }
            if (outputWriteStream) {
                asyncWriter.writeResponses(response, outputWriteStream);
            }
            resolve();
        })
        .catch(errorResponse => {
            recordInputArray = inputTextArray;

            showResponses(errorResponse.msg, ResponseTypeEnum.ERROR);
            if (outputWriteStream) {
                asyncWriter.writeError(errorResponse.payload, outputWriteStream);
            }
            resetSession();
            reject();
        });
    });
}

// When 1.error occurred, 2.skill session ended, we set shouldForceNewSession to ture 
// to make sure next utterance start a new skill session.
function resetSession() {
    inputTextArray = [];
    shouldForceNewSession = true;
}
